@@ -6,8 +6,10 @@ MIT License.  See Project Root for the license information.
 
 import os
 from typing import List, Optional, overload, Dict, Any
+from botocore.exceptions import ClientError
+from boto3.dynamodb.conditions import Attr
 
-from aws_lambda_powertools import Tracer, Logger
+from aws_lambda_powertools import Logger
 from boto3.dynamodb.conditions import (
     Key,
     # And,
@@ -15,14 +17,13 @@ from boto3.dynamodb.conditions import (
     ComparisonCondition,
     ConditionBase,
 )
-from boto3_assist.dynamodb.dynamodb_connection import DynamoDBConnection
-from boto3_assist.dynamodb.dynamodb_helpers import DynamoDBHelpers
-from boto3_assist.dynamodb.dynamodb_model_base import DynamoDBModelBase
-from boto3_assist.utilities.string_utility import StringUtility
-
+from .dynamodb_connection import DynamoDBConnection
+from .dynamodb_helpers import DynamoDBHelpers
+from .dynamodb_model_base import DynamoDBModelBase
+from ..utilities.string_utility import StringUtility
+from .dynamodb_index import DynamoDBIndex
 
 logger = Logger()
-tracer = Tracer()
 
 
 class DynamoDB(DynamoDBConnection):
@@ -41,6 +42,9 @@ class DynamoDB(DynamoDBConnection):
         aws_end_point_url: Optional[str] = None,
         aws_access_key_id: Optional[str] = None,
         aws_secret_access_key: Optional[str] = None,
+        assume_role_arn: Optional[str] = None,
+        assume_role_chain: Optional[List[str]] = None,
+        assume_role_duration_seconds: Optional[int] = 3600,
     ) -> None:
         super().__init__(
             aws_profile=aws_profile,
@@ -48,6 +52,9 @@ class DynamoDB(DynamoDBConnection):
             aws_end_point_url=aws_end_point_url,
             aws_access_key_id=aws_access_key_id,
             aws_secret_access_key=aws_secret_access_key,
+            assume_role_arn=assume_role_arn,
+            assume_role_chain=assume_role_chain,
+            assume_role_duration_seconds=assume_role_duration_seconds,
         )
         self.helpers: DynamoDBHelpers = DynamoDBHelpers()
         self.log_dynamodb_item_size = (
@@ -55,23 +62,27 @@ class DynamoDB(DynamoDBConnection):
         )
         logger.setLevel(os.getenv("LOG_LEVEL", "INFO"))
 
-    @tracer.capture_method
     def save(
         self,
         item: dict | DynamoDBModelBase,
         table_name: str,
         source: Optional[str] = None,
+        fail_if_exists: bool = False,
     ) -> dict:
         """
         Save an item to the database
         Args:
-            item (dict): DynamoDB Dictionay Object or DynamoDBModelBase.  Supports the "client" or
-            "resource" syntax
-            table_name (str): The DyamoDb Table Name
+            item (dict): DynamoDB Dictionary Object or DynamoDBModelBase.
+                Supports the "client" or "resource" syntax
+            table_name (str): The DynamoDb Table Name
             source (str, optional): The source of the call, used for logging. Defaults to None.
+            fail_if_exists (bool, optional): Only allow it to insert once.
+                Fail if it already exits. This is useful for loggers, historical records,
+                tasks, etc. that should only be created once
 
         Raises:
-            e: Any Error Raised
+            ClientError: Client specific errors
+            Exception: Any Error Raised
 
         Returns:
             dict: The Response from DynamoDB's put_item actions.
@@ -81,7 +92,7 @@ class DynamoDB(DynamoDBConnection):
 
         try:
             if not isinstance(item, dict):
-                # attemp to convert it
+                # attempt to convert it
                 if not isinstance(item, DynamoDBModelBase):
                     raise RuntimeError(
                         f"Item is not a dictionary or DynamoDBModelBase. Type: {type(item).__name__}. "
@@ -94,7 +105,7 @@ class DynamoDB(DynamoDBConnection):
                 except Exception as e:  # pylint: disable=w0718
                     logger.exception(e)
                     raise RuntimeError(
-                        "An error occured during model converation.  The entry was not saved. "
+                        "An error occurred during model conversion.  The entry was not saved. "
                     ) from e
 
             if isinstance(item, dict):
@@ -102,19 +113,49 @@ class DynamoDB(DynamoDBConnection):
 
             if isinstance(item, dict) and isinstance(next(iter(item.values())), dict):
                 # Use boto3.client syntax
-                response = dict(
-                    self.dynamodb_client.put_item(TableName=table_name, Item=item)
-                )
+                # client API style
+                params = {
+                    "TableName": table_name,
+                    "Item": item,
+                }
+                if fail_if_exists:
+                    # only insert if the item does *not* already exist
+                    params["ConditionExpression"] = (
+                        "attribute_not_exists(#pk) AND attribute_not_exists(#sk)"
+                    )
+                    params["ExpressionAttributeNames"] = {"#pk": "pk", "#sk": "sk"}
+                response = dict(self.dynamodb_client.put_item(**params))
+
             else:
                 # Use boto3.resource syntax
                 table = self.dynamodb_resource.Table(table_name)
-                response = dict(table.put_item(Item=item))  # type: ignore[arg-type]
+                if fail_if_exists:
+                    cond = Attr("pk").not_exists() & Attr("sk").not_exists()
+                    response = dict(table.put_item(Item=item, ConditionExpression=cond))
+                else:
+                    response = dict(table.put_item(Item=item))
+                    # response = dict(table.put_item(Item=item))  # type: ignore[arg-type]
+
+        except ClientError as e:
+            if (
+                fail_if_exists
+                and e.response["Error"]["Code"] == "ConditionalCheckFailedException"
+            ):
+                raise RuntimeError(
+                    f"Item with pk={item['pk']} already exists in {table_name} "
+                    f"and fail_if_exists was set to {fail_if_exists}"
+                ) from e
+
+            logger.exception(
+                {"source": f"{source}", "metric_filter": "put_item", "error": str(e)}
+            )
+            raise
 
         except Exception as e:  # pylint: disable=w0718
             logger.exception(
                 {"source": f"{source}", "metric_filter": "put_item", "error": str(e)}
             )
-            raise e
+            raise
 
         return response
 
@@ -169,7 +210,6 @@ class DynamoDB(DynamoDBConnection):
         call_type: str = "resource",
     ) -> Dict[str, Any]: ...
 
-    @tracer.capture_method
     def get(
         self,
         key: Optional[dict] = None,
@@ -276,11 +316,11 @@ class DynamoDB(DynamoDBConnection):
 
     def query(
         self,
-        key: dict | Key | ConditionBase | ComparisonCondition,
+        key: dict | Key | ConditionBase | ComparisonCondition | DynamoDBIndex,
+        table_name: str,
         *,
         index_name: Optional[str] = None,
         ascending: bool = False,
-        table_name: Optional[str] = None,
         source: Optional[str] = None,
         strongly_consistent: bool = False,
         projection_expression: Optional[str] = None,
@@ -302,9 +342,22 @@ class DynamoDB(DynamoDBConnection):
         """
 
         logger.debug({"action": "query", "source": source})
+        if not key:
+            raise ValueError("Query failed: key must be provided.")
+
+        if not table_name:
+            raise ValueError("Query failed: table_name must be provided.")
+
+        if isinstance(key, DynamoDBIndex):
+            if not index_name:
+                index_name = key.name
+            # turn it into a key expected by dynamodb
+            key = key.key(query_key=True)
 
         kwargs: dict = {}
-        if index_name:
+
+        if index_name and index_name != "primary":
+            # only include the index_name if we are not using our "primary" pk/sk
             kwargs["IndexName"] = f"{index_name}"
         kwargs["TableName"] = f"{table_name}"
         kwargs["KeyConditionExpression"] = key
@@ -327,7 +380,16 @@ class DynamoDB(DynamoDBConnection):
             raise ValueError("Query failed: table_name must be provided.")
 
         table = self.dynamodb_resource.Table(table_name)
-        response = dict(table.query(**kwargs))
+        response: dict = {}
+        try:
+            response = dict(table.query(**kwargs))
+        except Exception as e:  # pylint: disable=w0718
+            logger.exception(
+                {"source": f"{source}", "metric_filter": "query", "error": str(e)}
+            )
+            response = {"exception": str(e)}
+            if self.raise_on_error:
+                raise e
 
         return response
 
@@ -344,7 +406,6 @@ class DynamoDB(DynamoDBConnection):
     ) -> dict:
         pass
 
-    @tracer.capture_method
     def delete(
         self,
         *,
