@@ -25,10 +25,10 @@ class HasKeys(Protocol):
     """Interface for classes that have primary and sort keys"""
 
     def get_pk(self, index_name: str) -> Optional[str]:
-        """Inteface to get_pk"""
+        """Interface to get_pk"""
 
     def get_sk(self, index_name: str) -> Optional[str]:
-        """Inteface to get_sk"""
+        """Interface to get_sk"""
 
     def get_key(self, index_name: str) -> And | Equals:
         """Get the index name and key"""
