@@ -1,4 +1,8 @@
-"""Serialization Utility"""
+"""
+Geek Cafe, LLC
+Maintainers: Eric Wilson
+MIT License.  See Project Root for the license information.
+"""
 
 import datetime as dt
 import decimal
@@ -8,8 +12,8 @@ import uuid
 from datetime import datetime
 from decimal import Decimal
 from typing import Any, Dict, List, TypeVar
-
 from aws_lambda_powertools import Logger
+from boto3_assist.utilities.string_utility import StringUtility
 
 T = TypeVar("T")
 
@@ -18,7 +22,7 @@ logger = Logger()
 
 
 class SerializableModel:
-    """Library to Serialize object to a DynamoDB Format"""
+    """Library to Serialize object to a DynamoDB Format or other dictionary"""
 
     T = TypeVar("T", bound="SerializableModel")
 
@@ -43,9 +47,16 @@ class SerializableModel:
 
         return mapped
 
+    def dict(self) -> Dict[str, Any]:
+        """
+        Same as .to_dictionary
+
+        """
+        return self.to_dictionary()
+
     def to_dictionary(self) -> Dict[str, Any]:
         """
-        Convert the object to a dictionary.
+        Convert the object to a dictionary. Same as .dict()
         """
         # return Serialization.convert_object_to_dict(self)
         return Serialization.to_dict(
@@ -99,6 +110,112 @@ class JsonEncoder(json.JSONEncoder):
             )  # Or any other way you wish to serialize objects without __dict__
 
 
+class JsonConversions:
+    """
+    Json Conversion Utility
+    Used for snake_case to camelCase and vice versa
+    """
+
+    @staticmethod
+    def _camel_to_snake(value: str) -> str:
+        """Converts a camelCase to a snake_case"""
+        return StringUtility.camel_to_snake(value)
+
+    @staticmethod
+    def _snake_to_camel(value: str) -> str:
+        """Converts a value from snake_case to camelCase"""
+        return StringUtility.snake_to_camel(value)
+
+    @staticmethod
+    def _convert_keys(data, convert_func, deep: bool = True):
+        """
+        Recursively converts dictionary keys using the provided convert_func.
+
+        Parameters:
+        data: The input data (dict, list, or other) to process.
+        convert_func: Function to convert the keys (e.g. camel_to_snake or snake_to_camel).
+        deep (bool): If True (default), convert keys in all nested dictionaries.
+                    If False, only convert the keys at the current level.
+        """
+        if isinstance(data, dict):
+            new_dict = {}
+            for key, value in data.items():
+                new_key = convert_func(key)
+                # Only process nested structures if deep is True.
+                new_dict[new_key] = (
+                    JsonConversions._convert_keys(value, convert_func, deep)
+                    if deep
+                    else value
+                )
+            return new_dict
+        elif isinstance(data, list):
+            # For lists, if deep conversion is enabled, process each element.
+            return [
+                (
+                    JsonConversions._convert_keys(item, convert_func, deep)
+                    if deep
+                    else item
+                )
+                for item in data
+            ]
+        else:
+            return data
+
+    @staticmethod
+    def json_camel_to_snake(data, deep: bool = True):
+        """Converts all keys in the JSON structure from camelCase to snake_case.
+
+        Parameters:
+        data: The JSON-like structure (dict or list) to process.
+        deep (bool): If True, process keys in all nested dictionaries; if False, only at the first level.
+        """
+        return JsonConversions._convert_keys(
+            data, JsonConversions._camel_to_snake, deep
+        )
+
+    @staticmethod
+    def json_snake_to_camel(data, deep: bool = True):
+        """Converts all keys in the JSON structure from snake_case to camelCase.
+
+        Parameters:
+        data: The JSON-like structure (dict or list) to process.
+        deep (bool): If True, process keys in all nested dictionaries; if False, only at the first level.
+        """
+        return JsonConversions._convert_keys(
+            data, JsonConversions._snake_to_camel, deep
+        )
+
+    # # Example usage:
+    # if __name__ == "__main__":
+    #     sample_json = {
+    #         "firstName": "John",
+    #         "lastName": "Doe",
+    #         "address": {"streetAddress": "21 2nd Street", "city": "New York"},
+    #         "phoneNumbers": [
+    #             {"phoneType": "home", "phoneNumber": "2125551234"},
+    #             {"phoneType": "fax", "phoneNumber": "6465554567"},
+    #         ],
+    #     }
+
+    #     print("Original JSON:")
+    #     print(sample_json)
+
+    #     # Convert from camelCase to snake_case on all levels.
+    #     snake_json_deep = json_camel_to_snake(sample_json, deep=True)
+    #     print("\nConverted to snake_case (deep conversion):")
+    #     print(snake_json_deep)
+
+    #     # Convert from camelCase to snake_case only at the first level.
+    #     snake_json_shallow = json_camel_to_snake(sample_json, deep=False)
+    #     print("\nConverted to snake_case (first-level only):")
+    #     print(snake_json_shallow)
+
+    #     # Convert back from snake_case to camelCase on all levels.
+    #     camel_json_deep = json_snake_to_camel(snake_json_deep, deep=True)
+    #     print("\nConverted back to camelCase (deep conversion):")
+    #     print(camel_json_deep)
+
+
 class Serialization:
     """
     Serialization Class
@@ -265,7 +382,10 @@ class Serialization:
             setattr(target, "__actively_serializing_data__", True)
 
         for key, value in source.items():
-            if Serialization.has_attribute(target, key):
+            if isinstance(target, dict):
+                # our target is a dictionary, so we need to handle this differently
+                target[key] = value
+            elif Serialization.has_attribute(target, key):
                 attr = getattr(target, key)
                 expected_type = type(attr)
 
@@ -275,6 +395,9 @@ class Serialization:
                             if coerce:
                                 # Attempt to coerce the value to the expected type
                                 try:
+                                    if isinstance(value, list) and expected_type is str:
+                                        value = "".join(value)
+
                                     value = expected_type(value)
                                 except ValueError as e:
                                     logger.warning(
@@ -347,7 +470,7 @@ class Serialization:
                 "To work around this create a boolean (bool) property named __actively_serializing_data__. \n"
                 "e.g. self.__actively_serializing_data__: bool = False\n\n"
                 "Only issue/raise your exception if __actively_serializing_data__ is not True. \n\n"
-                "e.g. if not self.some_propert and not self.__actively_serializing_data__:\n"
+                "e.g. if not self.some_property and not self.__actively_serializing_data__:\n"
                 '    raise ValueError("some_property must be set")\n\n'
                 "This procedure will update the property from False to True while serializing, "
                 "then back to False once serialization is complete. "
